/*
 * Copyright (C) 2017 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "src/ftrace_reader/event_info.h"

namespace perfetto {
namespace {

Field FieldFromNameIdType(const char* name,
                          size_t id,
                          ProtoFieldType type,
                          FtraceFieldType ftrace_type) {
  Field field{};
  field.ftrace_name = name;
  field.ftrace_type = ftrace_type;
  field.proto_field_id = id;
  field.proto_field_type = type;
  return field;
}

}  // namespace

// TODO(hjd): Auto-generate this file.
// TODO(b/70373826): Reduce runetime overhead with constexpr magic etc.
std::vector<Event> GetStaticEventInfo() {
  std::vector<Event> events;

  {
    events.emplace_back(Event{});
    Event* event = &events.back();
    event->name = "print";
    event->group = "ftrace";
    event->proto_field_id = 3;
    event->fields.push_back(
        FieldFromNameIdType("buf", 2, kProtoString, kFtraceCString));
  }

  {
    events.emplace_back(Event{});
    Event* event = &events.back();
    event->name = "sched_switch";
    event->group = "sched";
    event->proto_field_id = 4;
    event->fields.push_back(
        FieldFromNameIdType("prev_comm", 1, kProtoString, kFtraceChar16));
    event->fields.push_back(
        FieldFromNameIdType("prev_pid", 2, kProtoUint32, kFtraceUint32));
    event->fields.push_back(
        FieldFromNameIdType("prev_prio", 3, kProtoUint32, kFtraceUint32));
    event->fields.push_back(
        FieldFromNameIdType("prev_state", 4, kProtoUint64, kFtraceUint64));
    event->fields.push_back(
        FieldFromNameIdType("next_comm", 5, kProtoString, kFtraceChar16));
    event->fields.push_back(
        FieldFromNameIdType("next_pid", 6, kProtoUint32, kFtraceUint32));
    event->fields.push_back(
        FieldFromNameIdType("next_prio", 7, kProtoUint32, kFtraceUint32));
  }

  return events;
}

bool SetTranslationStrategy(FtraceFieldType ftrace,
                            ProtoFieldType proto,
                            TranslationStrategy* out) {
  if (ftrace == kFtraceUint32 && proto == kProtoUint32) {
    *out = kUint32ToUint32;
  } else if (ftrace == kFtraceUint64 && proto == kProtoUint64) {
    *out = kUint64ToUint64;
  } else if (ftrace == kFtraceChar16 && proto == kProtoString) {
    *out = kChar16ToString;
  } else if (ftrace == kFtraceCString && proto == kProtoString) {
    *out = kCStringToString;
  } else {
    return false;
  }
  return true;
}

}  // namespace perfetto
